@@ -1,4 +1,5 @@
-use clap::{ArgEnum, Parser, Subcommand};
+use clap::{ArgEnum, Args, Parser, Subcommand};
+use miette::{IntoDiagnostic, Result, WrapErr};
 use std::{
     collections::HashSet,
     fs::{self},
@@ -7,6 +8,7 @@ use std::{
 use tinytemplate::TinyTemplate;
 
 mod comment_parser;
+mod errors;
 mod formats;
 mod git;
 mod toml_parser;
@@ -22,10 +24,19 @@ enum OutputFormat {
     Respec,
 }
 
-/// The different options that can be passed to this CLI
+/// To make cargo subcommands work, you need to use `bin_name`,
+/// as well as a subcommand.
 #[derive(Debug, Parser)]
+#[clap(bin_name = "cargo")]
+enum Cli {
+    #[clap(author, version, about)]
+    Spec(Opt),
+}
+
+/// The different options that can be passed to this CLI
+#[derive(Args, Debug)]
 #[clap(author, version, about, bin_name = "cargo")]
-struct Cli {
+struct Opt {
     /// The path to the specification toml file (defaults to Specification.toml).
     #[clap(short, long, parse(from_os_str), value_name = "SPEC_PATH")]
     specification_path: Option<PathBuf>,
@@ -59,9 +70,9 @@ enum Mode {
     CI,
 }
 
-fn main() {
+fn main() -> Result<()> {
     //~ 1. parse command-line arguments
-    let args = Cli::parse();
+    let Cli::Spec(args) = Cli::parse();
 
     let toml_spec = args
         .specification_path
@@ -77,8 +88,9 @@ fn main() {
     use Mode::*;
     match mode {
         //~   a. the `Build` mode builds the specification
-        Build => build(toml_spec.clone(), output_file.clone(), output_format)
-            .unwrap_or_else(|e| panic!("{}", e)),
+        Build => {
+            let _ = build(toml_spec.clone(), output_file.clone(), output_format)?;
+        }
         //~   b. the `Watch` mode builds the specification on every change
         Watch => {
             use notify::{watcher, RecursiveMode, Watcher};
@@ -146,6 +158,8 @@ fn main() {
             // check if what is built is the same as the result file, if not return an error (how to do exit(1) ?)
         }
     };
+
+    Ok(())
 }
 
 /// Builds the specification and returns a number of files to watch
@@ -153,12 +167,11 @@ fn build(
     toml_spec: PathBuf,
     output_file: Option<PathBuf>,
     output_format: OutputFormat,
-) -> Result<HashSet<PathBuf>, String> {
+) -> Result<HashSet<PathBuf>> {
     let mut files_to_watch = HashSet::new();
 
     //~ 3. parse the Specification.toml file
-    let mut specification =
-        toml_parser::parse_toml_spec(toml_spec.as_path()).map_err(|e| format!("{}", e))?;
+    let mut specification = toml_parser::parse_toml_spec(toml_spec.as_path())?;
     //    println!("specification: {:#?}", specification);
 
     let mut spec_dir =
@@ -166,12 +179,13 @@ fn build(
     spec_dir.pop();
 
     //~ 4. retrieve the template
-    let mut path = spec_dir.clone();
-    path.push(&specification.config.template);
-    files_to_watch.insert(path.clone());
+    let mut template_path = spec_dir.clone();
+    template_path.push(&specification.config.template);
+    files_to_watch.insert(template_path.clone());
 
-    let template =
-        fs::read_to_string(&path).map_err(|e| format!("could not read template file: {}", e))?;
+    let template = fs::read_to_string(&template_path)
+        .into_diagnostic()
+        .wrap_err_with(|| format!("could not read template {}", template_path.display(),))?;
 
     //~ 5. retrieve the content from all the files listed in the .toml
     for (_, filename) in &mut specification.sections {
@@ -179,25 +193,30 @@ fn build(
         path.push(&filename);
         files_to_watch.insert(path.clone());
 
-        *filename =
-            comment_parser::parse_file(path.to_str().expect("couldn't convert path to string"))
-                .map_err(|e| format!("{}", e))?;
+        *filename = comment_parser::parse_file(&path)?;
     }
 
     //~ 6. render the template
     let mut tt = TinyTemplate::new();
     tt.set_default_formatter(&tinytemplate::format_unescaped);
     tt.add_template("specification", &template)
-        .map_err(|e| format!("template file can't be parsed: {}", e))?;
+        .into_diagnostic()
+        .wrap_err_with(|| format!("can't parse template {}", template_path.display(),))?;
 
     let rendered = tt
         .render("specification", &specification)
-        .unwrap_or_else(|e| panic!("template file can't be rendered: {}", e));
+        .into_diagnostic()
+        .wrap_err_with(|| {
+            format!(
+                "template file can't be rendered: {}",
+                template_path.display()
+            )
+        })?;
 
     //~ 7. build the spec
     use OutputFormat::*;
     match output_format {
-        Markdown => formats::markdown::build(&specification, &rendered, output_file),
+        Markdown => formats::markdown::build(&rendered, output_file),
         Respec => {
             formats::respec::build(&specification, &rendered, output_file);
         }
