@@ -3,7 +3,8 @@ use std::collections::HashMap;
 use std::fs::File;
 use std::io::Read;
 use std::path::Path;
-use toml::de::Error;
+
+use miette::{IntoDiagnostic, Result, WrapErr};
 
 //~ spec:startcode
 /// A specification file contains a specification, as well as sections of (title, text)
@@ -38,10 +39,18 @@ pub struct Metadata {
 //~ spec:endcode
 
 /// Parse a `Specification.toml` file into a [Specification] struct.
-pub fn parse_toml_spec(spec_file: &Path) -> Result<Specification, Error> {
-    let mut file = File::open(spec_file).unwrap_or_else(|e| panic!("cannot open the specification file {}, make sure you pass a specification toml file via --specification-path", e));
+pub fn parse_toml_spec(spec_file: &Path) -> Result<Specification> {
+    let mut file = File::open(spec_file).into_diagnostic().wrap_err_with(|| format!("cannot open the specification file {}, make sure you pass a specification toml file via --specification-path", spec_file.display()))?;
+
     let mut content = String::new();
     file.read_to_string(&mut content)
-        .unwrap_or_else(|e| panic!("{}", e));
-    toml::from_str(&content)
+        .into_diagnostic()
+        .wrap_err_with(|| {
+            format!(
+                "couldn't read the specification file {}",
+                spec_file.display()
+            )
+        })?;
+
+    toml::from_str(&content).into_diagnostic()
 }
